@@ -11,6 +11,8 @@ translations = {
   "he":  "hebrew",
   "es":  "spanish",
   "it":  "italian",
+  "gl":  "galician",
+  "cz":  "czech",
 }
 
 files = ["fretsonfire", "tutorial"]
